using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using CrystalDecisions.CrystalReports.Engine;
using CrystalDecisions.Shared;

namespace BOE
{
  public partial class CrystalDocForm : Form
  {
    private ReportDocument rpt = null;
    private ReportDataSet rptDS = null;

    public CrystalDocForm()
    {
      InitializeComponent();
      rpt = new ReportDocument();
      rptDS = new ReportDataSet();
    }

    private void findRpt_Click(object sender, EventArgs e)
    {
      if (findRptFile.ShowDialog() == DialogResult.OK)
      {
        rptFileName.Text = findRptFile.FileName;
        rptFileName.Refresh();
        rptTitle.Text = "Loading report...";
        rptTitle.Refresh();
        rpt.Load(rptFileName.Text);
        if (rpt.SummaryInfo.ReportTitle != "")
          rptTitle.Text = rpt.SummaryInfo.ReportTitle;
        else
          rptTitle.Text = rptFileName.Text.Substring(rptFileName.Text.LastIndexOf('\\'));
        rptTitle.Refresh();
      }
    }

    private void docBtn_Click(object sender, EventArgs e)
    {
      //clear out the old results
      updateStatus("Clearing Tables");
      foreach (DataTable tbl in rptDS.Tables)
        tbl.Clear();

      //process main report

      rptDS.Report.AddReportRow(1, rptTitle.Text, rpt.DataDefinition.RecordSelectionFormula);
      processRpt(1, rpt);

      /* //process any subreports
      int rptNum = 2;
      Sections sections = rpt.ReportDefinition.Sections;
      foreach (Section section in sections)
      {
        ReportObjects reportObjects = section.ReportObjects;
        foreach (ReportObject reportObject in reportObjects)
        {
          if (reportObject.Kind == ReportObjectKind.SubreportObject)
          {
            SubreportObject subreportObject = (SubreportObject)reportObject;
            ReportDocument subReportDocument = subreportObject.OpenSubreport(subreportObject.SubreportName);
            rptDS.Report.AddReportRow(rptNum, subreportObject.SubreportName, subReportDocument.DataDefinition.RecordSelectionFormula);
            processRpt(rptNum, subReportDocument);
            rptNum++;
          }
        }
      }*/

      updateStatus("Generating Report");
      CrystalDocReportViewer viewer = new CrystalDocReportViewer(rptDS);
      viewer.Text = rptTitle.Text;
      viewer.Show(this);
      updateStatus("Done.");
    }

    private void processRpt(int rptNum, ReportDocument procRpt)
    {
      processFormulas(rptNum, procRpt.DataDefinition.FormulaFields);
      processRunningTotals(rptNum, procRpt.DataDefinition.RunningTotalFields);
      processGroups(rptNum, procRpt.DataDefinition.Groups);
      processSorts(rptNum, procRpt.DataDefinition.SortFields);
      processSQLExpressions(rptNum, procRpt.DataDefinition.SQLExpressionFields);
      processSummaries(rptNum, procRpt.DataDefinition.SummaryFields);
      processParams(rptNum, procRpt.DataDefinition.ParameterFields);
      processTables(rptNum, procRpt.Database.Tables);
      processLinks(rptNum, procRpt.Database.Links);
    }

    private void processFormulas(int rptNum, FormulaFieldDefinitions formulaDefs)
    {
      updateStatus("Processing Formulas");
      foreach (FormulaFieldDefinition formulaField in formulaDefs)
      {
        rptDS.Formulas.AddFormulasRow(rptNum, formulaField.FormulaName, formulaField.Text, formulaField.ValueType.ToString());
      }
    }

    private void processRunningTotals(int rptNum, RunningTotalFieldDefinitions rtFields)
    {
      updateStatus("Processing Running Totals");
      string evalCond;
      string resetCond;
      string condField;
      foreach (RunningTotalFieldDefinition rtField in rtFields)
      {
        if (rtField.EvaluationCondition == null)
          evalCond = string.Empty;
        else
          evalCond = rtField.EvaluationCondition.ToString();

        if (rtField.ResetCondition == null)
          resetCond = string.Empty;
        else
          resetCond = rtField.ResetCondition.ToString();

        if ((rtField.Group == null) || (rtField.Group.ConditionField == null))
          condField = string.Empty;
        else
          condField = rtField.Group.ConditionField.FormulaName;
        rptDS.RunningTotals.AddRunningTotalsRow(rptNum, rtField.FormulaName, rtField.SummarizedField.FormulaName,
                                                condField, rtField.Operation.ToString(),
                                                evalCond, resetCond);
      }
    }

    private void processGroups(int rptNum, Groups groups)
    {
      updateStatus("Processing Groups");
      Int16 groupNum = 1;
      foreach (Group group in groups)
      {
        rptDS.Groups.AddGroupsRow(rptNum, group.ConditionField.FormulaName, groupNum);
        groupNum++;
      }
    }

    private void processSorts(int rptNum, SortFields sortFields)
    {
      updateStatus("Processing Sorts");
      Int16 sortNum = 1;
      foreach (SortField sortField in sortFields)
      {
        rptDS.Sorts.AddSortsRow(rptNum, sortField.Field.FormulaName, sortField.SortDirection.ToString(), sortNum);
        sortNum++;
      }
    }

    private void processSQLExpressions(int rptNum, SQLExpressionFieldDefinitions sqlFields)
    {
      updateStatus("Processing SQL Expressions");
      foreach (SQLExpressionFieldDefinition sqlField in sqlFields)
      {
        rptDS.SQLExpressions.AddSQLExpressionsRow(rptNum, sqlField.FormulaName, sqlField.Text);
      }
    }

    private void processSummaries(int rptNum, SummaryFieldDefinitions summaryFields)
    {
      updateStatus("Processing Summaries");
      foreach (SummaryFieldDefinition summaryField in summaryFields)
      {
        rptDS.Summaries.AddSummariesRow(rptNum, summaryField.FormulaName, summaryField.SummarizedField.FormulaName,
                                        summaryField.Group.ConditionField.FormulaName, summaryField.Operation.ToString());
      }
    }

    private void processParams(int rptNum, ParameterFieldDefinitions paramFields)
    {
      //do nothing yet...
    }

    private void processTables(int rptNum, Tables tables)
    {
      updateStatus("Processing Tables");
      foreach (Table table in tables)
      {
        rptDS.TableList.AddTableListRow(rptNum, table.Name);
      }
    }

    private void processLinks(int rptNum, TableLinks links)
    {
      updateStatus("Processing Links");

      string srcFields;
      string destFields;
      foreach (TableLink link in links)
      {
        if (link.SourceFields == null)
          srcFields = string.Empty;
        else
          srcFields = getLinkFields(link.SourceFields);
        if (link.DestinationFields == null)
          destFields = string.Empty;
        else
          destFields = getLinkFields(link.DestinationFields);
        rptDS.TableLinks.AddTableLinksRow(rptNum, link.SourceTable.Name, link.DestinationTable.Name,
                                          link.JoinType.ToString(), srcFields, destFields);
      }
    }

    private string getLinkFields(DatabaseFieldDefinitions fields)
    {
      StringBuilder fieldList = new StringBuilder();
      foreach (DatabaseFieldDefinition field in fields)
      {
        if (fieldList.Length > 0)
          fieldList.Append(", " + field.Name);
        else
          fieldList.Append(field.Name);
      }
      return fieldList.ToString();
    }

    private void updateStatus(string msg)
    {
      statusLbl.Text = msg;
      statusLbl.Refresh();
    }

    private void closeBtn_Click(object sender, EventArgs e)
    {
      this.Close();
    }
  }
}